/************************************************************************/
/************         RR Stage Operand Multiplexor          *************/
/************************************************************************/
/*									*/
/* Module Name:	 	OPMUX                                    	*/
/* File Name:	 	irr_opmux.v                           		*/
/* Function:		Behavioral model of the Operand multiplexor	*/
/*			for the INTU		                        */
/*									*/
/* Language: 		Verilog						*/
/*									*/
/* Creator:		Andrew Chang  					*/
/* Creation date:	July 27th, 1994			        	*/
/* Last Mod by:		Andrew Chang  					*/
/*									*/
/*									*/
/* History:	940727	achang	Created					*/
/*									*/
/*		950402 achang  Extracted selection logic		*/
/*				from MUX merged with sext logic		*/
/*									*/
/*		951214 skeckler Changed mux input to mux in zero on     */
/*                              high bits when s1cc or s1gcc is         */
/*                              selected.  Same for s2cc, s2gcc         */
/*		960302 achang   Added IMUX_MSZPTR for setting MSZ and 	*/
/*				and PTR bits, eliminated K1 input	*/
/*				Renamed K2 input to be K		*/
/*									*/
/* Copyright (C) 1994, 1995, 1996, 1997 Andrew Chang                    */
/*                                                                      */
/* Massachusetts Institute of Technology                                */
/* Concurrent VLSI Architecture Group                                   */
/* All rights reserved                                                  */
/*									*/
/************************************************************************/

/*
  To Do:
    - remove immediate muxing into src1

*/

/* Interface

>>>>>>>>>>>>>>>> INPUT LIST <<<<<<<<<<<<<<<<<<<<<<<	
From OPSEL

	is1			Select src1 = irf data	
	is1cc				    = local cc data
	is1gcc				    = global cc data
	is1imm				    = immediate
	is1z				    = zero
	is1ip				    = ip

	is2			Select src2 = irf data	
	is2cc				    = local cc data
	is2gcc				    = global cc data
	is2imm				    = immediate
	is2z				    = zero
	is2ip				    = ip

	ms1			Select src1 = irf data	
	ms1cc				    = local cc data
	ms1gcc				    = global cc data
	ms1imm				    = immediate
	ms1z				    = zero
	ms1ip				    = ip

	ms2			Select src2 = irf data	
	ms2f				    = frf data
	ms2cc				    = local cc data
	ms2gcc				    = global cc data
	ms2imm				    = immediate
	ms2z				    = zero
	ms2ip				    = ip

	fs1			Select src1 = frf data	
	fs1imm				    = immediate
	fs1z				    = zero

	fs2			Select src2 = frf data	
	fs2imm				    = immediate
	fs2z				    = zero

	fs3			Select src2 = frf data	
	fs3z				    = zero

From SEXT

	ik,			INTU constant 
	mk1,mk2,		MEMU constant 
	fk1,fk2,		FPU  constant 

From IRF
	ird1, ird2,		INTU: src1, src2 register data
	
	mrd1, mrd2,		MEMU: src1, src2 register data

From FRF

	frd1, frd2, frd3,	FPU: src1, src2, src3 register data
	fmd,			MEMU: src2 register data

From CCRF

	ccd1,ccd2		INTU: ccsrc1, ccsrc2 data



>>>>>>>>>>>>>>>> OUTPUT LIST <<<<<<<<<<<<<<<<<<<<<<<	

To OPSZ

	iopr1, iopr2		INTU operands
	mopr1, mopr2		MEMU operands
	fopr1, fopr2		FPU operands

*/


/*==>>*/
module IMUXB 
	(
				      //from OPSEL	
	 s1,			      //Select src1 = irf data	
	 s1cc,			      //	    = local cc data
	 s1gcc,			      //	    = global cc data
	 s1pos,		      	      //	    = positive immediate
	 s1neg,		      	      //	    = negative immediate
	 s1z,			      //	    = zero
	 s1ip,			      //	    = ip
	 s2,			      //Select src2 = irf data	
	 s2cc,			      //	    = local cc data
	 s2gcc,			      //	    = global cc data
	 s2pos,		      	      //    	    = positive immediate
	 s2neg,		      	      //    	    = negative immediate 
	 s2z,			      //	    = zero
	 s2ip,			      //	    = ip
	 k,		      	      //from SEXT:  constant
	 rr_ip,		      	      //IP
	 rd1, rd2,	              //INTU: src1, src2 register data
	 lccd1,lccd2,		      //INTU: ccsrc1, ccsrc2 data
	 gccd1,gccd2,		      //INTU: ccsrc1, ccsrc2 data

	opr1, opr2		      //to OPZ:  INTU operands

	);

	input s1,s1cc,s1gcc;
	input s1pos, s1neg;
	input s1z,s1ip;
	input s2,s2cc,s2gcc;
	input s2pos, s2neg;
	input s2z,s2ip;
	input [`iMM_L-1:0] k;
	input [`WORD_W-1:0] rd1, rd2;
	input lccd1,lccd2,gccd1,gccd2;
	input [`WORD_W-1:0] rr_ip;
	
	output [`WORD_W-1:0] opr1, opr2;	//to OPZ:  INTU operands

	wire s1_b0, s2_b0;
	wire s1_b1, s2_b1;
	wire [15:0] s1_b17, s2_b17;
	wire [45:0] s1_b63, s2_b63;

	
/*<<==*/


/*************************************************************************/
/*	Operand Select:      output = X for undefined inputs  		 */
/*									 */
/*									 */
	assign #1
	  opr1 = {s1_b63,s1_b17,s1_b1,s1_b0},
	  opr2 = {s2_b63,s2_b17,s2_b1,s2_b0};



	assign #`TGPD
 	  s1_b0 =  s1pos ? 1'b0 :
 		    s1neg ? 1'b0  :
			s1z  ? 1'b0 : 
			    s1ip ? 1'b0:
				s1cc ? lccd1:
				  s1gcc ? gccd1:
					 s1 ? rd1[0]: 1'bx,
 	  s1_b1 =  s1pos ? 1'b0 :
 		    s1neg ? 1'b0  :
			s1z  ? 1'b0 : 
			    s1ip ? 1'b0:
				(s1cc | s1gcc) ? 1'b0:
					 s1 ? rd1[1]: 1'bx,
 	  s1_b17 =  s1pos ? 16'b0 :
 		      s1neg ? 16'b0 :
			s1z  ? 16'b0 : 
			    s1ip ? rr_ip[17:2]:
				(s1cc | s1gcc) ? 16'b0:
					 s1 ? rd1[17:2]: 16'bx,

 	  s1_b63 =  s1pos ? 46'b0 :
 		    s1neg ? 46'h3fffffffffff :
			s1z  ? 46'b0 : 
			    s1ip ? rr_ip[63:18]:
				(s1cc | s1gcc) ? 46'b0:
					 s1 ? rd1[63:18]: 46'bx;


	assign #`TGPD
 	  s2_b0 =  s2pos ? k[0] :
		    s2neg ? k[0] :
			s2z  ? 1'b0 : 
			    s2ip ? 1'b0 :
			        s2cc ? lccd2:
  			          s2gcc ? gccd2:
				         s2 ? rd2[0]: 1'bx,
 	  s2_b1 =  s2pos ? k[1] :
		    s2neg ? k[1] :
			s2z  ? 1'b0 : 
			    s2ip ? 1'b0:
				(s2cc | s2gcc) ? 1'b0:
				         s2 ? rd2[1]: 1'bx,
 	  s2_b17 =  s2pos ? k[17:2] :
		    s2neg ? k[17:2] :
			s2z  ? 16'b0 : 
			    s2ip ? rr_ip[17:2]:
				(s2cc | s2gcc) ? 16'b0:
				         s2 ? rd2[17:2]: 16'bx,
 	  s2_b63 =  s2pos ? 46'b0 :
		    s2neg ? 46'h3fffffffffff :
			s2z  ? 46'b0 : 
			    s2ip ? rr_ip[63:18]:
				(s2cc | s2gcc) ? 46'b0:
				         s2 ? rd2[63:18]: 46'bx;


endmodule






/*==>>*/
module IMUX_MSZPTR 				// MUX for MSZ and PTR
	(
				      //from OPSEL	
	 s1,			      //Select src1 = irf data	
	 s1cc,			      //	    = local cc data
	 s1gcc,			      //	    = global cc data
	 s1pos,		      	      //	    = positive immediate
	 s1neg,		      	      //	    = negative immediate
	 s1z,			      //	    = zero
	 s1ip,			      //	    = ip
	 s2,			      //Select src2 = irf data	
	 s2cc,			      //	    = local cc data
	 s2gcc,			      //	    = global cc data
	 s2pos,		      	      //    	    = positive immediate
	 s2neg,		      	      //    	    = negative immediate 
	 s2z,			      //	    = zero
	 s2ip,			      //	    = ip
	 rd1, rd2,	              //INTU: src1, src2 register data

	opr1, opr2		      //to OPZ:  INTU operands

	);

	input s1,s1cc,s1gcc;
	input s1pos, s1neg;
	input s1z,s1ip;
	input s2,s2cc,s2gcc;
	input s2pos, s2neg;
	input s2z,s2ip;
	input [`rFXTRA_W-1:0] rd1, rd2;
	
	output [`rFXTRA_W-1:0] opr1, opr2;	//to OPZ:  INTU operands

/*<<==*/


/*************************************************************************/
/*	Operand Select:      output = X for undefined inputs  		 */
/*									 */
/*									 */
	assign #`TGPD
 	  opr1 =  (s1pos | s1neg | s1z | s1cc | s1gcc) ? `rFXTRA_W'b10 :
	                 s1ip ? `rFXTRA_W'b11	:
					 s1 ? rd1: `rFXTRA_W'b0,
 	  opr2 =  (s2pos | s2neg | s2z | s2cc | s2gcc) ? `rFXTRA_W'b10 :
	                 s2ip ? `rFXTRA_W'b11	:
					s2 ? rd2: `rFXTRA_W'b0;

endmodule





